/**
 * 
 */
package net.bigeon.gclc.process;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import org.junit.Test;

import net.bigeon.gclc.command.ICommandProvider;
import net.bigeon.gclc.command.SubedCommand;
import net.bigeon.gclc.exception.CommandRunException;
import net.bigeon.gclc.exception.CommandRunExceptionType;
import net.bigeon.gclc.exception.InvalidCommandName;
import net.bigeon.gclc.process.mocks.CommandMock;

/**
 * @author Emmanuel Bigeon
 *
 */
public class CommandForkTest {

    /**
     * Test method for {@link net.bigeon.gclc.process.CommandFork#CommandFork(java.lang.String, net.bigeon.gclc.command.ICommandProvider, net.bigeon.gclc.process.TaskPool)}.
     */
    @Test
    public void testCommandForkStringICommandProviderTaskPool() {
        final TaskPool pool = new TaskPool();
        final ICommandProvider provider = new SubedCommand("test");
        final CommandFork pl = new CommandFork("fork", provider, pool);
        assertEquals("Command name should be preserved", "fork", pl.getCommandName());
    }

    /** Test method for
     * {@link net.bigeon.gclc.process.CommandFork#execute(net.bigeon.gclc.manager.ConsoleOutput, net.bigeon.gclc.manager.ConsoleInput, java.lang.String[])}.
     * 
     * @throws InvalidCommandName if the test init failed
     * @throws CommandRunException if the command execution failed in the test */
    @Test
    public void testExecute() throws InvalidCommandName, CommandRunException {
        final TaskPool pool = new TaskPool(false);
        final ICommandProvider provider = new SubedCommand("test");
        final CommandFork pl = new CommandFork("fork", provider, pool);

        try {
            pl.execute(null, null);
            fail("No command specified should through an exception");
        } catch (final CommandRunException e) {
            assertEquals("No command specified is a usage error",
                    CommandRunExceptionType.USAGE, e.getType());
        }
        try {
            pl.execute(null, null, "invalid");
            fail("Invalid command specified should through an exception");
        } catch (final CommandRunException e) {
            assertEquals("Invalid command specified is a usage error",
                    CommandRunExceptionType.USAGE, e.getType());
        }

        final CommandMock mock = new CommandMock();
        provider.add(mock);

        pl.execute(null, null, mock.getCommandName());

        final ForkTask task = (ForkTask) pool.get(pool.getPIDs().iterator().next());
        task.join(null, null, 1000);

        assertEquals("Command should be executed when forked", 1, mock.getExecuteCall());
    }

    /**
     * Test method for {@link net.bigeon.gclc.process.CommandFork#tip()}.
     */
    @Test
    public void testTip() {
        final TaskPool pool = new TaskPool();
        final ICommandProvider provider = new SubedCommand("test");
        final CommandFork pl = new CommandFork("fork", provider, pool);
        assertNotNull("Command tip should be defined", pl.tip());
        assertNotNull("Command usage should be defined", pl.usagePattern());
        assertNotNull("Command usage should be defined", pl.usageDetail());
    }

}
